@@ -39,7 +39,7 @@ C<transfer> will perl stacks (a few k) and optionally C stack (4-16k) be
 allocated. On systems supporting mmap a 128k stack is allocated, on the
 assumption that the OS has on-demand virtual memory. All this is very
 system-dependent. On my i686-pc-linux-gnu system this amounts to about 10k
-per coroutine.
+per coroutine, 5k when the experimental context sharing is enabled.
 
 =over 4
 
@@ -48,7 +48,7 @@ per coroutine.
 package Coro::State;
 
 BEGIN {
-   $VERSION = 0.12;
+   $VERSION = 0.13;
 
    require XSLoader;
    XSLoader::load Coro::State, $VERSION;
@@ -70,9 +70,18 @@ to save the current coroutine in.
 
 =cut
 
+# this is called (or rather: goto'ed) for each and every
+# new coroutine. IT MUST NEVER RETURN and should not call
+# anything that changes the stacklevel (like eval).
 sub initialize {
    my $proc = shift;
-   &$proc while 1;
+   eval {
+      &$proc while 1;
+   };
+   if ($@) {
+      print STDERR "FATAL: uncaught exception\n$@";
+   }
+   _exit 255;
 }
 
 sub new {
@@ -124,19 +133,6 @@ This exists mainly to ease subclassing (wether through @ISA or not).
 
 =cut
 
-=item $error->($error_coro, $error_msg)
-
-This function will be called on fatal errors. C<$error_msg> and
-C<$error_coro> return the error message and the error-causing coroutine
-(NOT an object) respectively. This API might change.
-
-=cut
-
-$error = sub {
-   print STDERR "FATAL: $_[1]\n";
-   exit 51;
-};
-
 =item Coro::State::flush
 
 To be efficient (actually, to not be abysmaly slow), this module does
